@@ -6,8 +6,9 @@ use 5.026;
 use utf8;
 use warnings;
 use open qw(:std :utf8);
+use experimental qw(signatures);
 
-our $VERSION = '0.013';
+our $VERSION = '0.014';
 
 =encoding utf8
 
@@ -30,7 +31,7 @@ Call it as a program with a name, character, or hex code number:
 		character   ✓
 
 	% perl lib/App/unichar.pm ✓
-	Processing CHECK MARK
+	Processing ✓
 		match type  grapheme
 		code point  U+2713
 		decimal     10003
@@ -52,17 +53,17 @@ values of characters.
 
 =head1 AUTHOR
 
-brian d foy, C<bdfoy@cpan.org>.
+brian d foy, C<briandfoy@pobox.com>.
 
 =head1 SOURCE AVAILABILITY
 
 This module is in Github:
 
-	https://github.com/briandfoy/unichar
+	https://github.com/briandfoy/app-unichar
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2011-2021 brian d foy
+Copyright 2011-2024 brian d foy
 
 This module is licensed under the Artistic License 2.0.
 
@@ -92,57 +93,70 @@ my $codeset = langinfo(CODESET);
 
 run( @ARGV ) unless caller;
 
-sub run {
-	foreach ( @ARGV ) {
-		my $fallthrough = 1;
+sub run (@args) {
+	foreach ( @args ) {
 		say "Processing $_";
-		my( $code, $match );
 
 		if( / \A (?: $r{u} | $r{h} | $r{d} ) \z /x ) {
-			$match = 'code point';
 			my( $key ) = keys %+;
-			$code = $transformation{$key}( $+{$key} );
-			$fallthrough = 0;
+			my $code = $transformation{$key}( $+{$key} );
+			output( $code, 'code point' );
 			}
-		if( / \A ([A-Z\s]{2,}) \z /ix ) {
-			$match = 'name';
-			$code = eval { charnames::vianame( uc($1) ) };
+		elsif( / \A ([A-Z\s]{2,}) \z /ix ) {
+			my $code = eval { charnames::vianame( uc($1) ) };
 			unless( defined $code ) {
 				say "\tCouldn't match <$1> to a code name";
 				next;
 				}
-			$fallthrough = 0;
+			output( $code, 'name' );
 			}
-		if( / \A (\X) \z /x ) {
-			$match = 'grapheme';
-			$code = ord( $1 );
-			$fallthrough = 0;
+		elsif( / \A (\X) \z /x ) {
+			output( ord( $1 ), 'grapheme' );
 			}
+		elsif( / \A r: ([A-Z\s]{2,}) \z /ix ) { # new regex mode
+			state $names = name_list();
+			say "In elsif";
+			my $pattern = s/\Ar://r;
+			$pattern = eval{ qr/$pattern/i };
+			if( $@ ) {
+			    warn "Invalid pattern --> $pattern ---> $@\n";
+			    exit(4);
+			    }
 
-		if( $fallthrough ) {
-			unless( $code ) {
-				say "\tInvalid character or codepoint --> $_\n";
+			foreach my $name ( keys $names->%* ) {
+				say "Tring $name";
+				next unless $name =~ m/$pattern/;
+				output( $names->{$name}, 'pattern' );
 				}
+			}
+		else {
+			say "\tInvalid character, codepoint, or pattern --> $_\n";
 			next;
 			}
-
-		my $hex  = sprintf 'U+%04X', $code;
-		my $char = chr( $code );
-		$char = '<unprintable>' if $char !~ /\p{Print}/;
-		$char = '<whitespace>'  if $char =~ /\p{Space}/;
-		$char = '<control>'     if $char =~ /\p{Control}/;
-
-		my $name = charnames::viacode( $code ) // '<no name found>';
-
-		print <<~"HERE";
-			match type  $match
-			code point  $hex
-			decimal     $code
-			name        $name
-			character   $char
-
-		HERE
-
 		}
 	}
 
+sub name_list () {
+	state $names = { map { charnames::viacode($_), $_ } 0 .. 0x3FFFF };
+	return $names;
+	}
+
+sub output ( $code, $match ) {
+	my $hex  = sprintf 'U+%04X', $code;
+	my $char = chr( $code );
+	$char = '<unprintable>' if $char !~ /\p{Print}/;
+	$char = '<whitespace>'  if $char =~ /\p{Space}/;
+	$char = '<control>'     if $char =~ /\p{Control}/;
+
+	my $name = charnames::viacode( $code ) // '<no name found>';
+
+	print <<~"HERE";
+		match type  $match
+		code point  $hex
+		decimal     $code
+		name        $name
+		character   $char
+
+	HERE
+
+	}
